/*!
 * @ignorance/vue-validator 0.0.2 (https://github.com/yesixuan/@ignorance/vue-validator)
 * API https://github.com/yesixuan/@ignorance/vue-validator/blob/master/doc/api.md
 * Copyright 2017-2018 yesixuan. All Rights Reserved
 * Licensed under MIT (https://github.com/yesixuan/@ignorance/vue-validator/blob/master/LICENSE)
 */

import 'core-js/modules/web.dom.iterable';
import 'core-js/modules/es6.object.keys';
import 'core-js/modules/es6.array.reduce';
import 'core-js/modules/es6.array.for-each';
import 'core-js/modules/es6.array.some';
import 'core-js/modules/es6.regexp.constructor';
import 'core-js/modules/es6.function.bind';
import 'core-js/modules/es6.function.name';

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

var regexpMap = {
  mobile: /^1\d{10}$/,
  // 手机校验
  url: /^((https|http|ftp|rtsp|mms)?:\/\/)[^\s]+/,
  // 网址校验
  identificationCard: /^\d{13,17}[0-9xX]$/,
  // 身份证号码格式校验
  tel: /(^(\d{3,4}-)?\d{7,8})$|(^1\d{10}$)/ // 校验电话号码

};

var Rules =
/*#__PURE__*/
function () {
  function Rules(regexpMap) {
    _classCallCheck(this, Rules);

    this.regexpMap = regexpMap;
    this.rules = {};
    this.createRegexpRule();
  }
  /**
   * 根据正则创建键值相同的校验方法
   */


  _createClass(Rules, [{
    key: "createRegexpRule",
    value: function createRegexpRule() {
      var _this = this;

      Object.keys(this.regexpMap).reduce(function (res, currentValue) {
        res[currentValue] = function (val) {
          return _this.regexpMap[currentValue].test(val + '');
        };

        return res;
      }, this.rules);
    }
    /**
     * 允许用户传入正则表达式来扩展
     */

  }, {
    key: "extendRegexp",
    value: function extendRegexp(regMap) {
      this.regexpMap = _objectSpread({}, this.regexpMap, regMap);
      this.createRegexpRule();
    }
    /**
     * 允许用户传入校验方法来扩展
     */

  }, {
    key: "extendValidator",
    value: function extendValidator(validatorMap) {
      this.rules = _objectSpread({}, this.rules, validatorMap);
    }
  }]);

  return Rules;
}();

var defaultRules = new Rules(regexpMap);

// console.log(rules)

var Validator =
/*#__PURE__*/
function () {
  function Validator(el, _ref, _ref2, _Vue) {
    var _this = this;

    var arg = _ref.arg,
        value = _ref.value,
        _ref$value = _ref.value,
        fields = _ref$value.fields,
        rules = _ref$value.rules,
        modifiers = _ref.modifiers;
    var context = _ref2.context;

    _classCallCheck(this, Validator);

    this.changeListener = function (e) {
      _this.verifySingle(e.target.value, _this.rules[e.target.name], e.target.name);
    };

    this.focusListener = function (_ref3) {
      var target = _ref3.target;

      if (target.nodeName === 'INPUT' || target.nodeName === 'SELECT' || target.nodeName === 'TEXTAREA') {
        _this.vm.$set(_this.$vec, target.name, {
          pass: true
        }); // this.validateData[target.name] = { pass: true }


        _this.prevTarget = target;
      }
    };

    this.submitListener = function (e) {
      e.preventDefault();

      if (_this.autoCatch) {
        var _this$checkAll = _this.checkAll(),
            pass = _this$checkAll.pass;

        if (pass) {
          _this.submitMethod();
        }
      } else {
        _this.submitMethod();
      }
    };

    this.blurListener = function (e) {
      if (e.target !== _this.prevTarget && _this.prevTarget !== null) {
        _this.verifySingle(_this.prevTarget.value, _this.rules[_this.prevTarget.name], _this.prevTarget.name);

        _this.vm.$forceUpdate();
      }
    };

    this.vm = context;
    this._Vue = _Vue; // this.inputValue = value

    this.ref = context.$refs[value.ref];
    this.ref.validator = this.checkAll.bind(this); // 让 vue 组件本身可以校验所有数据

    this.formData = context[value.formData];
    this.fields = fields;
    this.rules = rules; // this.validateData = {} // 存放校验信息的对象

    this.autoCatch = !!modifiers.autoCatch;
    this.submitMethod = this.vm[arg];
    this.prevTarget = null;
    this.createReactiveData();
    this.initEvent(el); // this.initPassData()
  }
  /* 初始化响应式对象 */


  _createClass(Validator, [{
    key: "createReactiveData",
    value: function createReactiveData() {
      var data = this.fields.reduce(function (res, key) {
        return _objectSpread({}, res, _defineProperty({}, key, {
          pass: true
        }));
      }, {});

      this._Vue.util.defineReactive(this.vm._data, '$vec', data);
    }
  }, {
    key: "createRegValidator",

    /**
     * 验证 validator 的值类型，将其统一包装成函数
     * @param validator
     * @returns Function
     */
    value: function createRegValidator(validator) {
      if (typeof validator === 'string') {
        try {
          return defaultRules.rules[validator];
        } catch (e) {
          throw "\u60A8\u8FD8\u672A\u5B9A\u4E49 ".concat(validator, " \u8FD9\u6761\u89C4\u5219");
        }
      } else if (validator instanceof RegExp) {
        return function (val) {
          return validator.test(val);
        };
      } else if (typeof validator === 'function') {
        return validator;
      } else {
        throw 'validator 的值只能为函数或正则表达式';
      }
    }
  }, {
    key: "createValidateData",
    value: function createValidateData(target, name, res) {
      target = res;
      this.vm.$set(this.$vec, name, target); // this.validateData[name] = target
    }
    /**
     * 创建错误信息数据的偏函数（提前接收两个固定参数）
     * @param args
     * @returns {function(*=): void}
     */

  }, {
    key: "partialCreateValidateData",
    value: function partialCreateValidateData() {
      var _this2 = this;

      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return function (res) {
        return _this2.createValidateData.apply(_this2, args.concat([res]));
      };
    }
    /**
     * 检验某个字段的校验是否通过
     * @param val
     * @param rules
     * @param name
     * @returns {*}
     */

  }, {
    key: "verifySingle",
    value: function verifySingle(val, rules, name) {
      var res = {};
      var required = rules.some(function (rule) {
        return rule.validator === 'required';
      });
      var handler = this.partialCreateValidateData(res, name);

      for (var i = 0; i < rules.length; i++) {
        var rule = rules[i];

        if (val === '' && !required) {
          handler({
            pass: true,
            msg: '校验通过'
          });
          return res;
        }

        if (required && val === '') {
          handler({
            pass: false,
            msg: '必填'
          });
          return res;
        }

        if (val !== '' && rule.validator !== 'required' && !this.createRegValidator(rule.validator)(val)) {
          handler({
            pass: false,
            msg: rule.msg || '默认校验不通过消息'
          });
          return res;
        }
      }

      handler({
        pass: true,
        msg: '校验通过'
      });
      return res;
    }
    /**
     * 提交的时候校验所有表单
     * @returns {{pass: boolean}}
     */

  }, {
    key: "checkAll",
    value: function checkAll() {
      var _this3 = this;

      var res = {
        pass: true
      };
      this.fields.forEach(function (item) {
        var checkOne = _this3.verifySingle(_this3.formData[item], _this3.rules[item], item);

        if (!checkOne.pass && res.pass) {
          res = _objectSpread({}, checkOne, {
            name: item
          });
        }
      });
      this.vm.$forceUpdate();
      return res;
    }
    /**
     * 这样定义方法才能保证该方法被添作事件监听者的时候 this 的指向符合预期
     * @param e
     */

  }, {
    key: "initEvent",
    value: function initEvent(el) {
      this.ref.addEventListener('change', this.changeListener); // 当鼠标聚焦时，这个表单元素需要正常

      this.ref.addEventListener('click', this.focusListener, true); // 绑定提交事件

      el.addEventListener('click', this.submitListener); // 模拟 blur 事件

      window.addEventListener('click', this.blurListener, true);
    }
  }, {
    key: "unbindEvent",
    value: function unbindEvent(el) {
      this.ref.removeEventListener('change', this.changeListener);
      this.ref.removeEventListener('click', this.focusListener, true);
      el.removeEventListener('click', this.submitListener);
      window.removeEventListener('click', this.blurListener, true);
    }
  }, {
    key: "$vec",
    get: function get() {
      return this.vm._data.$vec;
    }
  }]);

  return Validator;
}();
var rules = defaultRules;

var validate = (function (ValidatePlugin, _Vue) {
  return {
    inserted: function inserted(el, binding, vNode) {
      ValidatePlugin.validators instanceof Array || (ValidatePlugin.validators = []);
      ValidatePlugin.createValidator(ValidatePlugin, el, binding, vNode, _Vue);
      vNode.context.isFormComponent = true;
    },
    unbind: function unbind(el, binding, _ref) {
      var context = _ref.context;

      // 解绑事件
      ValidatePlugin.validators[context._uid].unbindEvent(el); // 通过组件的 id 来区分不同的表单


      ValidatePlugin.validators[context._uid] = null;
    }
  };
});

function verify (name) {
  return this._data.$vec ? !this._data.$vec[name].pass : false;
}

var ValidatePlugin =
/*#__PURE__*/
function () {
  function ValidatePlugin() {
    _classCallCheck(this, ValidatePlugin);

    throw new Error('不允许实例化 ValidatePlugin');
  }

  _createClass(ValidatePlugin, null, [{
    key: "createValidator",
    value: function createValidator(ValidatePlugin, el, binding, vNode, _Vue) {
      ValidatePlugin.validators[vNode.context._uid] = new Validator(el, binding, vNode, _Vue);
    }
  }, {
    key: "install",
    value: function install(_Vue, options) {
      if (!ValidatePlugin.validation) {
        ValidatePlugin.validation = {}; // 存放所有校验实例的容器
      }

      _Vue.directive('validate', validate(ValidatePlugin, _Vue)); // 可供组件直接调用的校验方法


      _Vue.prototype.$verify = verify;
    }
  }]);

  return ValidatePlugin;
}();

export default ValidatePlugin;
export { rules };
